@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2012, Albertas Vyšniauskas
+ * Copyright (c) 2009-2021, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -18,12 +18,10 @@
 
 #include "ColorWheelType.h"
 #include "ColorRYB.h"
-#include "MathUtil.h"
-#include "Internationalisation.h"
-
+#include "I18N.h"
 
 static void rgb_hue2hue(double hue, Color* hsl){
-	hsl->hsl.hue = hue;
+	hsl->hsl.hue = static_cast<float>(hue);
 	hsl->hsl.saturation = 1;
 	hsl->hsl.lightness = 0.5;
 }
@@ -35,7 +33,7 @@ static void rgb_rgbhue2hue(double rgbhue, double *hue){
 static void ryb1_hue2hue(double hue, Color* hsl){
 	Color c;
 	color_rybhue_to_rgb(hue, &c);
-	color_rgb_to_hsl(&c, hsl);
+	*hsl = c.rgbToHsl();
 }
 
 static void ryb1_rgbhue2hue(double rgbhue, double *hue){
@@ -43,7 +41,7 @@ static void ryb1_rgbhue2hue(double rgbhue, double *hue){
 }
 
 static void ryb2_hue2hue(double hue, Color* hsl){
-	hsl->hsl.hue = color_rybhue_to_rgbhue_f(hue);
+	hsl->hsl.hue = static_cast<float>(color_rybhue_to_rgbhue_f(hue));
 	hsl->hsl.saturation = 1;
 	hsl->hsl.lightness = 0.5;
 }
@@ -65,4 +63,3 @@ const ColorWheelType* color_wheel_types_get(){
 const uint32_t color_wheel_types_get_n(){
 	return sizeof(color_wheel_types)/sizeof(ColorWheelType);
 }
-
