@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2012, Albertas Vyšniauskas
+ * Copyright (c) 2009-2021, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -16,8 +16,8 @@
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef NOISE_H_
-#define NOISE_H_
+#ifndef GPICK_NOISE_H_
+#define GPICK_NOISE_H_
 
 /** \file source/Noise.h
  * \brief Improved Perlin noise implementation.
@@ -32,5 +32,4 @@
  */
 double noise(double x, double y, double z);
 
-#endif /* NOISE_H_ */
-
+#endif /* GPICK_NOISE_H_ */
