@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2012, Albertas Vyšniauskas
+ * Copyright (c) 2009-2021, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -16,52 +16,37 @@
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef LAYOUT_STYLE_H_
-#define LAYOUT_STYLE_H_
-
+#ifndef GPICK_LAYOUT_STYLE_H_
+#define GPICK_LAYOUT_STYLE_H_
 #include "../Color.h"
-
 #include "ReferenceCounter.h"
 #include "Box.h"
-
 #include <string>
 #include <list>
-
-namespace layout{
-
-class Box;
-
-class Style:public ReferenceCounter{
-public:
+namespace layout {
+struct Box;
+struct Style: public ReferenceCounter {
 	std::string ident_name;
-	std::string human_name;
-
+	std::string label;
 	Color color;
 	float font_size;
-
-	enum{
-		TYPE_UNKNOWN = 0,
-		TYPE_COLOR,
-		TYPE_BACKGROUND,
-		TYPE_BORDER,
-	}style_type;
-
+	enum class Type {
+		unknown = 0,
+		color,
+		background,
+		border,
+	};
+	Type styleType;
 	bool dirty;
-
 	bool highlight;
-	Box* selected_box;
-
-	bool IsDirty();
-	void SetDirty(bool dirty);
-
-	bool GetHighlight();
-	Box* GetBox();
-	void SetState(bool highlight, Box *box);
-
-	Style(const char* name, Color* color, float font_size);
+	Box *selected_box;
+	bool isDirty();
+	void setDirty(bool dirty);
+	bool getHighlight();
+	Box *getBox();
+	void setState(bool highlight, Box *box);
+	Style(const char *name, Color *color, float font_size);
 	virtual ~Style();
 };
-
 }
-
 #endif /* LAYOUT_STYLE_H_ */
