@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2012, Albertas Vyšniauskas
+ * Copyright (c) 2009-2016, Albertas Vyšniauskas
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
@@ -17,13 +17,14 @@
  */
 
 #include "PaletteFromImage.h"
-#include "../uiUtilities.h"
-#include "../uiListPalette.h"
-#include "../GlobalStateStruct.h"
-#include "../ToolColorNaming.h"
-#include "../DynvHelpers.h"
-#include "../Internationalisation.h"
-
+#include "ColorList.h"
+#include "ColorObject.h"
+#include "uiUtilities.h"
+#include "uiListPalette.h"
+#include "GlobalState.h"
+#include "ToolColorNaming.h"
+#include "dynv/Map.h"
+#include "I18N.h"
 #include <string.h>
 #include <iostream>
 #include <sstream>
@@ -40,68 +41,61 @@ using namespace std;
  *
  * Each node can b
  */
-typedef struct Node{
-	uint32_t n_pixels;          /**< Number of colors in current Node and its children */
-	uint32_t n_pixels_in;       /**< Number of colors in current Node */
-	float color[3];             /**< Sum of color values */
-	float distance;             /**< Squared distances from Node center of colors in Node */
-
-	Node *child[8];             /**< Pointers to child Nodes */
-	Node *parent;               /**< Pointer to parent Node */
-}Node;
+struct Node{
+	uint32_t n_pixels; /**< Number of colors in current Node and its children */
+	uint32_t n_pixels_in; /**< Number of colors in current Node */
+	float color[3]; /**< Sum of color values */
+	float distance; /**< Squared distances from Node center of colors in Node */
+	Node *child[8]; /**< Pointers to child Nodes */
+	Node *parent; /**< Pointer to parent Node */
+};
 
 /** \struct Cube
  * \brief Cube structure holds all information necessary to define cube size and position in space
  */
-typedef struct Cube{
-	float x;                 /**< X position */
-	float w;                 /**< Width */
-	float y;                 /**< Y position */
-	float h;                 /**< Height */
-	float z;                 /**< Z position */
-	float d;                 /**< Depth */
-}Cube;
+struct Cube{
+	float x; /**< X position */
+	float w; /**< Width */
+	float y; /**< Y position */
+	float h; /**< Height */
+	float z; /**< Z position */
+	float d; /**< Depth */
+};
 
-typedef struct PaletteFromImageArgs{
+struct PaletteFromImageArgs{
 	GtkWidget *file_browser;
 	GtkWidget *range_colors;
 	GtkWidget *merge_threshold;
 	GtkWidget *preview_expander;
-
 	string filename;
 	uint32_t n_colors;
-
 	string previous_filename;
 	Node *previous_node;
-
-	struct ColorList *color_list;
-	struct ColorList *preview_color_list;
-
-	struct dynvSystem *params;
+	ColorList *color_list;
+	ColorList *preview_color_list;
+	dynv::Ref options;
 	GlobalState* gs;
-}PaletteFromImageArgs;
+};
 
-class PaletteColorNameAssigner: public ToolColorNameAssigner {
-	protected:
-		stringstream m_stream;
-		const char *m_filename;
-		int m_index;
-	public:
-		PaletteColorNameAssigner(GlobalState *gs):ToolColorNameAssigner(gs){
-			m_index = 0;
-		}
-
-		void assign(struct ColorObject *color_object, Color *color, const char *filename, const int index){
-			m_filename = filename;
-			m_index = index;
-			ToolColorNameAssigner::assign(color_object, color);
-		}
-
-		virtual std::string getToolSpecificName(struct ColorObject *color_object, Color *color){
-			m_stream.str("");
-			m_stream <<  m_filename << " #" << m_index;
-			return m_stream.str();
-		}
+struct PaletteColorNameAssigner: public ToolColorNameAssigner {
+	PaletteColorNameAssigner(GlobalState &gs):
+		ToolColorNameAssigner(gs) {
+		m_index = 0;
+	}
+	void assign(ColorObject &colorObject, std::string_view fileName, const int index) {
+		m_fileName = fileName;
+		m_index = index;
+		ToolColorNameAssigner::assign(colorObject);
+	}
+	virtual std::string getToolSpecificName(const ColorObject &colorObject) override {
+		m_stream.str("");
+		m_stream << m_fileName << " #" << m_index;
+		return m_stream.str();
+	}
+protected:
+	std::stringstream m_stream;
+	std::string_view m_fileName;
+	int m_index;
 };
 
 /**
@@ -343,7 +337,7 @@ static Node* process_image(PaletteFromImageArgs *args, const char *filename, Nod
 		args->previous_node = 0;
 	}
 
-	GError *error = NULL;
+	GError *error = nullptr;
 	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file(filename, &error);
 	if (error){
 		cout << error->message << endl;
@@ -374,24 +368,20 @@ static Node* process_image(PaletteFromImageArgs *args, const char *filename, Nod
 		ptr = image_data + rowstride * y;
 		for (int x = 0; x < width; x++){
 
-			color.xyz.x = ptr[0] / 255.0;
-			color.xyz.y = ptr[1] / 255.0;
-			color.xyz.z = ptr[2] / 255.0;
+			color.xyz.x = ptr[0] / 255.0f;
+			color.xyz.y = ptr[1] / 255.0f;
+			color.xyz.z = ptr[2] / 255.0f;
 
 			node_update(args->previous_node, &color, &cube, 5);
 
 			ptr += channels;
 		}
 	}
-
 	g_object_unref(pixbuf);
-
 	node_reduce(args->previous_node, 200);
-
 	return node_copy(args->previous_node, 0);
 }
 
-
 static void get_settings(PaletteFromImageArgs *args){
 
 	gchar *filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(args->file_browser));
@@ -402,20 +392,20 @@ static void get_settings(PaletteFromImageArgs *args){
 		args->filename.clear();
 	}
 
-	args->n_colors = gtk_spin_button_get_value(GTK_SPIN_BUTTON(args->range_colors));
+	args->n_colors = static_cast<int>(gtk_spin_button_get_value(GTK_SPIN_BUTTON(args->range_colors)));
 }
 
 static void save_settings(PaletteFromImageArgs *args){
-	dynv_set_int32(args->params, "colors", args->n_colors);
+	args->options->set("colors", static_cast<int32_t>(args->n_colors));
 	gchar *current_folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(args->file_browser));
 	if (current_folder){
-		dynv_set_string(args->params, "current_folder", current_folder);
+		args->options->set("current_folder", current_folder);
 		g_free(current_folder);
 	}
 	GtkFileFilter *filter = gtk_file_chooser_get_filter(GTK_FILE_CHOOSER(args->file_browser));
 	if (filter){
 		const char *filter_name = static_cast<const char*>(g_object_get_data(G_OBJECT(filter), "name"));
-		dynv_set_string(args->params, "filter", filter_name);
+		args->options->set("filter", filter_name);
 	}
 }
 
@@ -424,16 +414,16 @@ static void calc(PaletteFromImageArgs *args, bool preview, int limit){
 	Node *root_node = 0;
 	int index = 0;
 	gchar *name = g_path_get_basename(args->filename.c_str());
-	PaletteColorNameAssigner name_assigner(args->gs);
+	PaletteColorNameAssigner name_assigner(*args->gs);
 	if (!args->filename.empty())
 		root_node = process_image(args, args->filename.c_str(), root_node);
 
-	struct ColorList *color_list;
+	ColorList *color_list;
 
 	if (preview)
 		color_list = args->preview_color_list;
 	else
-		color_list = args->gs->colors;
+		color_list = args->gs->getColorList();
 
 	list<Color> tmp_list;
 
@@ -444,34 +434,25 @@ static void calc(PaletteFromImageArgs *args, bool preview, int limit){
 	}
 
 	for (list<Color>::iterator i = tmp_list.begin(); i != tmp_list.end(); i++){
-		struct ColorObject *color_object = color_list_new_color_object(color_list, &(*i));
-		name_assigner.assign(color_object, &(*i), name, index);
+		i->alpha = 1;
+		ColorObject *color_object = color_list_new_color_object(color_list, &(*i));
+		name_assigner.assign(*color_object, name, index);
 		color_list_add_color_object(color_list, color_object, 1);
-		color_object_release(color_object);
+		color_object->release();
 		index++;
 	}
-
 }
 
-
 static void update(GtkWidget *widget, PaletteFromImageArgs *args ){
 	color_list_remove_all(args->preview_color_list);
 	get_settings(args);
 	calc(args, true, 100);
 }
 
-
-static gchar* format_threshold_value_cb(GtkScale *scale, gdouble value){
-	return g_strdup_printf("%0.01f%%", value);
-}
-
 static void destroy_cb(GtkWidget* widget, PaletteFromImageArgs *args){
 
 	if (args->previous_node) node_delete(args->previous_node);
-
 	color_list_destroy(args->preview_color_list);
-	dynv_system_release(args->params);
-
 	delete args;
 }
 
@@ -483,9 +464,9 @@ static void response_cb(GtkWidget* widget, gint response_id, PaletteFromImageArg
 	gint width, height;
 	gtk_window_get_size(GTK_WINDOW(widget), &width, &height);
 
-	dynv_set_int32(args->params, "window.width", width);
-	dynv_set_int32(args->params, "window.height", height);
-	dynv_set_bool(args->params, "show_preview", gtk_expander_get_expanded(GTK_EXPANDER(args->preview_expander)));
+	args->options->set("window.width", width);
+	args->options->set("window.height", height);
+	args->options->set<bool>("show_preview", gtk_expander_get_expanded(GTK_EXPANDER(args->preview_expander)));
 
 	switch (response_id){
 		case GTK_RESPONSE_APPLY:
@@ -499,26 +480,19 @@ static void response_cb(GtkWidget* widget, gint response_id, PaletteFromImageArg
 	}
 }
 
-
-void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs){
-
+void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs)
+{
 	PaletteFromImageArgs *args = new PaletteFromImageArgs;
-
-    args->previous_filename = "";
+	args->previous_filename = "";
 	args->gs = gs;
-	args->params = dynv_get_dynv(args->gs->params, "gpick.tools.palette_from_image");
+	args->options = args->gs->settings().getOrCreateMap("gpick.tools.palette_from_image");
 	args->previous_node = 0;
-
 	GtkWidget *table, *table_m, *widget;
-
-	GtkWidget *dialog = gtk_dialog_new_with_buttons(_("Palette from image"), parent, GtkDialogFlags(GTK_DIALOG_DESTROY_WITH_PARENT), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, GTK_STOCK_ADD, GTK_RESPONSE_APPLY, NULL);
-
-	gtk_window_set_default_size(GTK_WINDOW(dialog), dynv_get_int32_wd(args->params, "window.width", -1),
-		dynv_get_int32_wd(args->params, "window.height", -1));
-
+	GtkWidget *dialog = gtk_dialog_new_with_buttons(_("Palette from image"), parent, GtkDialogFlags(GTK_DIALOG_DESTROY_WITH_PARENT), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, GTK_STOCK_ADD, GTK_RESPONSE_APPLY, nullptr);
+	gtk_window_set_default_size(GTK_WINDOW(dialog), args->options->getInt32("window.width", -1),
+		args->options->getInt32("window.height", -1));
 	gtk_dialog_set_alternative_button_order(GTK_DIALOG(dialog), GTK_RESPONSE_APPLY, GTK_RESPONSE_CLOSE, -1);
 
-
 	GtkWidget *frame;
 	gint table_y, table_m_y;
 
@@ -534,29 +508,28 @@ void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs){
 	gtk_container_add(GTK_CONTAINER(frame), table);
 
 	args->file_browser = widget = gtk_file_chooser_button_new(_("Image file"), GTK_FILE_CHOOSER_ACTION_OPEN);
-	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(widget), dynv_get_string_wd(args->params, "current_folder", ""));
+	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(widget), args->options->getString("current_folder", "").c_str());
 	gtk_table_attach(GTK_TABLE(table), widget, 0, 3, table_y, table_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	g_signal_connect(G_OBJECT(args->file_browser), "file-set", G_CALLBACK(update), args);
 	table_y++;
 
-
-	const char* selected_filter = dynv_get_string_wd(args->params, "filter", "all_images");
+	auto selected_filter = args->options->getString("filter", "all_images");
 	GtkFileFilter *filter;
 	GtkFileFilter *all_image_filter;
 
 	filter = gtk_file_filter_new();
 	gtk_file_filter_set_name(filter, _("All files"));
 	gtk_file_filter_add_pattern(filter, "*");
-	g_object_set_data_full(G_OBJECT(filter), "name", (void*)"all_files", GDestroyNotify(NULL));
+	g_object_set_data_full(G_OBJECT(filter), "name", (void*)"all_files", GDestroyNotify(nullptr));
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(widget), filter);
-	if (g_strcmp0("all_files", selected_filter) == 0) gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(widget), filter);
+	if ("all_files" == selected_filter) gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(widget), filter);
 
 
 	all_image_filter = gtk_file_filter_new();
 	gtk_file_filter_set_name(all_image_filter, _("All images"));
-	g_object_set_data_full(G_OBJECT(all_image_filter), "name", (void*)"all_images", GDestroyNotify(NULL));
+	g_object_set_data_full(G_OBJECT(all_image_filter), "name", (void*)"all_images", GDestroyNotify(nullptr));
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(widget), all_image_filter);
-	if (g_strcmp0("all_images", selected_filter) == 0) gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(widget), all_image_filter);
+	if ("all_images" == selected_filter) gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(widget), all_image_filter);
 
 	stringstream ss;
 	GSList *formats = gdk_pixbuf_get_formats();
@@ -568,24 +541,24 @@ void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs){
 		gtk_file_filter_set_name(filter, gdk_pixbuf_format_get_description(format));
 
 		gchar **extensions = gdk_pixbuf_format_get_extensions(format);
-        if (extensions){
+		if (extensions){
 			for (int j = 0; extensions[j]; j++){
 				ss.str("");
 				ss << "*." << extensions[j];
-				gtk_file_filter_add_pattern(filter, ss.str().c_str());
-				gtk_file_filter_add_pattern(all_image_filter, ss.str().c_str());
+				auto pattern = ss.str();
+				gtk_file_filter_add_pattern(filter, pattern.c_str());
+				gtk_file_filter_add_pattern(all_image_filter, pattern.c_str());
 			}
 			g_strfreev(extensions);
 		}
-		g_object_set_data_full(G_OBJECT(filter), "name", gdk_pixbuf_format_get_name(format), GDestroyNotify(NULL));
+		g_object_set_data_full(G_OBJECT(filter), "name", gdk_pixbuf_format_get_name(format), GDestroyNotify(nullptr));
 		gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(widget), filter);
-		if (g_strcmp0(gdk_pixbuf_format_get_name(format), selected_filter) == 0) gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(widget), filter);
+		if (gdk_pixbuf_format_get_name(format) == selected_filter) gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(widget), filter);
 
 		i = g_slist_next(i);
 	}
 	if (formats) g_slist_free(formats);
 
-
 	frame = gtk_frame_new(_("Options"));
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
 	gtk_table_attach(GTK_TABLE(table_m), frame, 0, 1, table_m_y, table_m_y+1, GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL), 5, 5);
@@ -594,40 +567,24 @@ void tools_palette_from_image_show(GtkWindow* parent, GlobalState* gs){
 	table_y=0;
 	gtk_container_add(GTK_CONTAINER(frame), table);
 
-
-
 	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Colors:"),0,0,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
 	args->range_colors = widget = gtk_spin_button_new_with_range (1, 100, 1);
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(args->range_colors), dynv_get_int32_wd(args->params, "colors", 3));
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(args->range_colors), args->options->getInt32("colors", 3));
 	gtk_table_attach(GTK_TABLE(table), widget,1,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
 	g_signal_connect(G_OBJECT(args->range_colors), "value-changed", G_CALLBACK(update), args);
 	table_y++;
 
-
-    /*
-	gtk_table_attach(GTK_TABLE(table), gtk_label_aligned_new(_("Threshold:"),0,0,0,0),0,1,table_y,table_y+1,GtkAttachOptions(GTK_FILL),GTK_FILL,5,5);
-	args->merge_threshold = widget = gtk_hscale_new_with_range(0, 100, 0.1);
-	gtk_range_set_value(GTK_RANGE(widget), dynv_get_float_wd(args->params, "merge_threshold", 20));
-	g_signal_connect(G_OBJECT(widget), "value-changed", G_CALLBACK(update), args);
-	g_signal_connect(G_OBJECT(widget), "format-value", G_CALLBACK(format_threshold_value_cb), args);
-	gtk_scale_set_value_pos(GTK_SCALE(widget), GTK_POS_RIGHT);
-	gtk_table_attach(GTK_TABLE(table), widget,1,3,table_y,table_y+1,GtkAttachOptions(GTK_FILL | GTK_EXPAND),GTK_FILL,3,3);
-	table_y++;
-    */
-
-	struct ColorList* preview_color_list = NULL;
-	gtk_table_attach(GTK_TABLE(table_m), args->preview_expander = palette_list_preview_new(gs, true, dynv_get_bool_wd(args->params, "show_preview", true), gs->colors, &preview_color_list), 0, 1, table_m_y, table_m_y+1 , GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL | GTK_EXPAND), 5, 5);
+	ColorList* preview_color_list = nullptr;
+	gtk_table_attach(GTK_TABLE(table_m), args->preview_expander = palette_list_preview_new(gs, true, args->options->getBool("show_preview", true), gs->getColorList(), &preview_color_list), 0, 1, table_m_y, table_m_y+1 , GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL | GTK_EXPAND), 5, 5);
 	table_m_y++;
 
 	args->preview_color_list = preview_color_list;
 
 	gtk_widget_show_all(table_m);
-	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table_m);
+	setDialogContent(dialog, table_m);
 
 	g_signal_connect(G_OBJECT(dialog), "destroy", G_CALLBACK(destroy_cb), args);
 	g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(response_cb), args);
 
 	gtk_widget_show(dialog);
 }
-
-
