@@ -1,29 +1,13 @@
-#!/usr/bin/env python
-
-import os
-import time
-import SCons
-import re
-import string
-import sys
-import glob
-import subprocess
-
-from lemon import *
-from flex import *
-from gettext import *
-from resource_template import *
-
-from SCons.Script import *
-from SCons.Util import *
+import os, time, re, string, glob, subprocess
+from .gettext import *
+from .ragel import *
+from .template import *
+from .version import *
+from SCons.Script import Chmod, Flatten
+from SCons.Util import NodeList
 from SCons.Script.SConscript import SConsEnvironment
 
-import SCons.Script.SConscript
-import SCons.SConf
-import SCons.Conftest
-
 def MatchFiles (files, path, repath, dir_exclude_pattern,  file_exclude_pattern):
-
 	for filename in os.listdir(path):
 		fullname = os.path.join (path, filename)
 		repath_file =  os.path.join (repath, filename);
@@ -49,18 +33,40 @@ def CheckProgram(context, env, name, member_name):
 		context.Result(False)
 		return False
 
+def CompareVersions(a, b):
+	for i in range(0, min(len(a), len(b))):
+		if a[i] < b[i]:
+			return 1
+		if a[i] > b[i]:
+			return -1
+	return 0
+
+def CheckBoost(context, version):
+	context.Message('Checking for library boost >= %s... ' % (version))
+	result, boost_version = context.TryRun("""
+#include <boost/version.hpp>
+#include <iostream>
+int main(){
+std::cout << BOOST_LIB_VERSION << std::endl;
+return 0;
+}
+""", '.cpp')
+	if result:
+		found_version = boost_version.strip('\r\n\t ').split('_')
+		required_version = version.strip('\r\n\t ').split('.')
+		result = CompareVersions(required_version, found_version) >= 0
+	context.Result(result)
+	return result
+
 class GpickLibrary(NodeList):
 	include_dirs = []
 
 class GpickEnvironment(SConsEnvironment):
-	
 	extern_libs = {}
-	
 	def AddCustomBuilders(self):
-		addLemonBuilder(self)
-		addFlexBuilder(self)
 		addGettextBuilder(self)
-		addResourceTemplateBuilder(self)
+		addTemplateBuilder(self)
+		addRagelBuilder(self)
 		
 	def DefineLibrary(self, library_name, library):
 		self.extern_libs[library_name] = library
@@ -79,9 +85,9 @@ class GpickEnvironment(SConsEnvironment):
 	def ConfirmPrograms(self, conf, programs):
 		conf.AddTests({'CheckProgram': CheckProgram})
 		
-		for evar, args in programs.iteritems():
+		for evar, args in programs.items():
 			found = False
-			for name, member_name in args['checks'].iteritems():
+			for name, member_name in args['checks'].items():
 				if conf.CheckProgram(self, name, member_name):
 					found = True;
 					break
@@ -95,9 +101,9 @@ class GpickEnvironment(SConsEnvironment):
 	def ConfirmLibs(self, conf, libs):
 		conf.AddTests({'CheckPKG': CheckPKG})
 		
-		for evar, args in libs.iteritems():
+		for evar, args in libs.items():
 			found = False
-			for name, version in args['checks'].iteritems():
+			for name, version in args['checks'].items():
 				if conf.CheckPKG(name + ' ' + version):
 					self[evar]=name
 					found = True;
@@ -109,6 +115,13 @@ class GpickEnvironment(SConsEnvironment):
 				else:
 					self.Exit(1)
 
+	def ConfirmBoost(self, conf, version):
+		conf.AddTests({'CheckBoost': CheckBoost})
+		if conf.CheckBoost(version):
+			return
+		else:
+			self.Exit(1)
+
 	def InstallPerm(self, dir, source, perm):
 		obj = self.Install(dir, source)
 		for i in obj:
@@ -127,9 +140,9 @@ class GpickEnvironment(SConsEnvironment):
 				self.AddPostAction(i, Chmod(i, perm))
 		return dir
 
-	InstallProgram = lambda self, dir, source: GpickEnvironment.InstallPerm(self, dir, source, 0755)
-	InstallData = lambda self, dir, source: GpickEnvironment.InstallPerm(self, dir, source, 0644)
-	InstallDataAutoDir = lambda self, dir, relative_dir, source: GpickEnvironment.InstallPermAutoDir(self, dir, relative_dir, source, 0644)
+	InstallProgram = lambda self, dir, source: GpickEnvironment.InstallPerm(self, dir, source, 0o755)
+	InstallData = lambda self, dir, source: GpickEnvironment.InstallPerm(self, dir, source, 0o644)
+	InstallDataAutoDir = lambda self, dir, relative_dir, source: GpickEnvironment.InstallPermAutoDir(self, dir, relative_dir, source, 0o644)
 
 	def GetSourceFiles(self, dir_exclude_pattern, file_exclude_pattern):
 		dir_exclude_prog = re.compile(dir_exclude_pattern)
@@ -140,27 +153,26 @@ class GpickEnvironment(SConsEnvironment):
 
 	def GetVersionInfo(self):
 		try:
-			revision = subprocess.Popen(['hg', 'log', '--template', '"{rev}:{node}\\n"', '-r', 'tip',  self.GetLaunchDir()], shell=False, stdout=subprocess.PIPE, stderr=subprocess.STDOUT).communicate()[0]
-			match = re.search('([\d]+):([\d\w]+)', str(revision))
-			revision = match.group(2)
+			(version, revision, hash, date) = getVersionInfo()
 		except:
-			revision = 'not under version control system'
-
-		self.Replace(GPICK_BUILD_REVISION = revision,
-			GPICK_BUILD_DATE =  time.strftime ("%Y-%m-%d"),
-			GPICK_BUILD_TIME =  time.strftime ("%H:%M:%S"));	
+			try:
+				with open("../.version", "r", encoding = 'utf-8') as version_file:
+					(version, revision, hash, date) = version_file.read().splitlines()
+			except:
+				print("Version file \".version\" is required when GIT can not be used to get version information.")
+				self.Exit(1)
+		self.Replace(
+			GPICK_BUILD_VERSION = version,
+			GPICK_BUILD_REVISION = revision,
+			GPICK_BUILD_HASH = hash,
+			GPICK_BUILD_DATE = date,
+			GPICK_BUILD_VERSION_FULL = (version + '-' + revision) if int(revision) > 0 else version,
+			GPICK_BUILD_VERSION_FULL_COMMA = re.sub(r'(\d+)[^\.]*', '\\1', version).replace('.', ',') + ',' + revision,
+		);
 
 def RegexEscape(str):
 	return str.replace('\\', '\\\\')
 
-def WriteNsisVersion(target, source, env):
-	for t in target:
-		for s in source:
-			file = open(str(t),"w")
-			file.writelines('!define VERSION "' + str(env['GPICK_BUILD_VERSION']) + '"')
-			file.close()
-	return 0
-
 def Glob(path):
 	files = []
 	for f in glob.glob(os.path.join(path, '*')):
@@ -170,4 +182,3 @@ def Glob(path):
 			files.append(str(f));
 	return files
 
-
